﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2011/04/20 初版
//      : 2011/11/10 ２版 リストに対する表示イベントを「MouseUp」に変更
//
// Circle Hough Sample Program
// -----------------------------------------------------------------
namespace Circle_Hough_Sample
{
    public partial class Form_CIRCLE_HOUGH_SAMPLE : Form
    {
        //画像メモリ格納領域
        private FVIL.Data.CFviImage m_image = new FVIL.Data.CFviImage();
        //ビデオ入力クラス
        private FVIL.Video.CFviVideo m_video = null;
        //円ハフ検出クラス(相関エッジ法)
        private FVIL.Hough.CFviCircleHoughCorrelationEdge m_houghcorrelation = new FVIL.Hough.CFviCircleHoughCorrelationEdge();
        //円ハフ検出クラス(ソーベルエッジ法)
        private FVIL.Hough.CFviCircleHoughSobelEdge m_houghsobel = new FVIL.Hough.CFviCircleHoughSobelEdge();
        //パラメータ設定 (本体)
        private FVIL.Hough.CFviCircleHoughParam m_param;
        //パラメータ設定 (エッジ検出部分)
        private FVIL.Edge.CFviEdge2DCorrelationParam m_correlationparam;
        //パラメータ設定 (エッジ検出部分)
        private FVIL.Edge.CFviEdge2DSobelParam m_sobelparam;

        //表示用オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlay = new FVIL.GDI.CFviOverlay();
        //エッジ表示用オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlayEdge = new FVIL.GDI.CFviOverlay();
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //iniファイル設定フォーム
        Form_BROWSEINI formBroseIni = new Form_BROWSEINI();

        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;

        //時間測定用
        private double msec = 0;

        public Form_CIRCLE_HOUGH_SAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            //使用許諾の確認
            dialogLicence licence_dialog = new dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

            // 描画imageのセット
            cFviImageView1.Image = m_image;
            //オーバーレイの追加
            cFviImageView1.Display.Overlays.Add(m_overlay);
            cFviImageView1.Display.Overlays.Add(m_overlayEdge);
        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;

            //計測時間テキストボックス初期化
            textBoxTime.Text = "0";

            //パラメータ初期化
            m_correlationparam = m_houghcorrelation.ParamEdge;
            m_sobelparam = m_houghsobel.ParamEdge;

            //トリガモードコンボボックス初期化
            comboBoxTrigger.Items.Clear();
            comboBoxTrigger.Items.Add("ノーマル");
            comboBoxTrigger.Items.Add("内部トリガ");
            comboBoxTrigger.Items.Add("外部トリガ");
            comboBoxTrigger.SelectedIndex = 0;

            //円ハフ検出フィルタ選択コンボボックス初期化
            comboBoxSelectFilter.Items.Clear();
            comboBoxSelectFilter.Items.Add("相関エッジ");
            comboBoxSelectFilter.Items.Add("ソーベル");
            comboBoxSelectFilter.SelectedIndex = 0;

            //検出する円の色コンボボックス初期化
            comboBoxObjectColor.Items.Clear();
            comboBoxObjectColor.Items.Add("黒い円");
            comboBoxObjectColor.Items.Add("白い円");
            comboBoxObjectColor.Items.Add("円の色が未知");
            comboBoxObjectColor.SelectedIndex = 2;
        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイルの選択
            m_strIniFilePath = "";
            if (formBroseIni.ShowDialog(this) == DialogResult.OK)
            {
                m_strIniFilePath = formBroseIni.strIniFilePath;
            }

            if (m_strIniFilePath.Length > 0)
            {
                try
                {
                    //設定ファイルのロードとデバイスオープン
                    m_video = FVIL.Video.Function.CreateInstance(m_strIniFilePath);
                    //画像サイズ調整
                    m_video.ChangeImageSize(m_image);
                    m_flgVideoOpened = true;

                    //ステータス表示の更新
                    //トリガモードの取得
                    comboBoxTrigger.SelectedIndex = (int)m_video.Trigger;

                    //カメラ露光時間の取得
                    if (m_video.DeviceType != FVIL.Video.DeviceType.RICE001)
                    {
                        numericUpDownExposure.Value = m_video.ExposureTime;
                    }
                    else
                    {
                        tabControlParam.TabPages.Remove(tabPageBoard);
                    }

                    buttonGrabOne.Enabled = true;
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else if (formBroseIni.UseFileFlg == true)
            {
                buttonGrabOne.Enabled = false;
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void buttonGrabOne_Click(object sender, EventArgs e)
        {
            try
            {
                m_overlay.Figures.Clear();
                formBroseIni.UseFileFlg = false;

                m_video.ChangeImageSize(m_image);

                //デバイスタイプ
                FVIL.Video.CFviContinuousGrab contgrab;
                switch (m_video.DeviceType)
                {
                    case FVIL.Video.DeviceType.FHC3312:
                        contgrab = ((FVIL.Video.CFviVideoFHC3312)m_video).ContinuousGrab;
                        ((FVIL.Video.CFviContinuousGrabFHC3312)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    case FVIL.Video.DeviceType.FHC3313:
                        contgrab = ((FVIL.Video.CFviVideoFHC3313)m_video).ContinuousGrab;
                        ((FVIL.Video.CFviContinuousGrabFHC3313)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    default:
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        m_video.GrabImageSync(m_image);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                }
                //測定時間表示
                textBoxTime.Text = msec.ToString("0.000");

                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void comboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.Trigger = (uint)comboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void numericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.ExposureTime = (uint)numericUpDownExposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// LoadFile ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 入力画像ファイルのロード
        /// </remarks>
        private void buttonLoadImageFile_Click(object sender, EventArgs e)
        {
            //描画クリア
            m_overlay.Figures.Clear();
            listViewResult.Items.Clear();

            formBroseIni.UseFileFlg = true;
            //入力画像ファイルのロード
            openFileDialog1.FileName = null;
            openFileDialog1.Filter = "Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            openFileDialog1.Filter += "|Png ファイル (*.png)|*.png";
            openFileDialog1.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            openFileDialog1.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            openFileDialog1.DefaultExt = "*.bmp";
            if ((openFileDialog1.FileNames.Length <= 0) && (openFileDialog1.ShowDialog(this) == DialogResult.Cancel))
            {
                return;
            }

            try
            {
                FVIL.File.Function.LoadImageFile(openFileDialog1.FileName, m_image, FVIL.PixelMode.Unpacking);
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// 円ハフ検出実行ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 円ハフ検出処理・結果表示
        /// </remarks>
        private void buttonDetection_Click(object sender, EventArgs e)
        {
            //表示初期化
            m_overlay.Figures.Clear();
            listViewResult.Items.Clear();

            //再描画
            cFviImageView1.Refresh();

            //パラメータ設定
            if (fnCircleHoughPraramSet())
            {
                //円ハフ検出実行
                if (fnCircleHoughExec())
                {
                    //結果表示
                    fnResultDisplay();
                }
            }
        }

        /// <summary>
        /// パラメータ設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パラメータ設定関数
        /// </remarks>
        private bool fnCircleHoughPraramSet()
        {
            //パラメータ設定
            if (comboBoxSelectFilter.SelectedIndex == 0)
            {
                //相関エッジパラメータ設定
                m_param = m_houghcorrelation.Param;
            }
            else
            {
                //ソーベルエッジパラメータ設定
                m_param = m_houghsobel.Param;
            }
            m_param.Resolution = Double.Parse(textBoxResolution.Text.ToString());
            m_param.RadiusMin = Double.Parse(textBoxRadiusMin.Text.ToString());
            m_param.RadiusMax = Double.Parse(textBoxRadiusMax.Text.ToString());
            m_param.QRange = (int)numericUpDownQRange.Value;
            m_param.EdgePercent = (int)numericUpDownEdgePercent.Value;
            m_param.MinScore = (int)numericUpDownMinScore.Value;
            switch (comboBoxObjectColor.SelectedIndex)
            {
                case 0:
                    m_param.ObjectColor = FVIL.Hough.ObjectColor.Black;
                    break;
                case 1:
                    m_param.ObjectColor = FVIL.Hough.ObjectColor.White;
                    break;
                case 2:
                    m_param.ObjectColor = FVIL.Hough.ObjectColor.BlackOrWhite;
                    break;
            }
            m_param.MergeCenter = (int)numericUpDownMergeCenter.Value;
            m_param.MergeRadius = (int)numericUpDownMergeRadius.Value;

            //エッジ検出部分パラメータ設定
            if (comboBoxSelectFilter.SelectedIndex == 0)
            {
                //相関エッジパラメータ設定
                m_correlationparam = m_houghcorrelation.ParamEdge;
                int temp1 = (int)numericUpDownRegionHeight.Value;
                int temp2 = (int)numericUpDownRegionWidth.Value;
                if ((temp1 <= temp2) && ((temp1 * temp2) <= 255))
                {
                    m_correlationparam.RegionWidth = (int)numericUpDownRegionWidth.Value;
                    m_correlationparam.RegionHeight = (int)numericUpDownRegionHeight.Value;
                }
                else
                {
                    MessageBox.Show("局所領域サイズの条件を満たしていません", "Parameter Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return (false);
                }
                m_correlationparam.SigmoidK = Double.Parse(textBoxSigmoidK.Text.ToString());
                m_correlationparam.VarThreshold = Double.Parse(textBoxVarThreshold.Text.ToString());
                m_correlationparam.EdgeThreshold = (int)numericUpDownEdgeThreshold.Value;
                m_correlationparam.NmsLength = (int)numericUpDownNmsLength.Value;
                //ボーダー処理
                if (checkBoxBorderFlg.Checked == true)
                {
                    //端延長モード
                    m_houghcorrelation.BorderMode = FVIL.BorderMode.Continuous;
                }
                else
                {
                    //ボーダー拡張しない
                    m_houghcorrelation.BorderMode = FVIL.BorderMode.None;
                }
            }
            else
            {
                //ソーベルエッジパラメータ設定
                m_sobelparam = m_houghsobel.ParamEdge;
                m_sobelparam.EdgeThreshold = (int)numericUpDownEdgeThreshold.Value;
                m_sobelparam.NmsLength = (int)numericUpDownNmsLength.Value;
                //ボーダー処理
                if (checkBoxBorderFlg.Checked == true)
                {
                    //端延長モード
                    m_houghsobel.BorderMode = FVIL.BorderMode.Continuous;
                }
                else
                {
                    //ボーダー拡張しない
                    m_houghsobel.BorderMode = FVIL.BorderMode.None;
                }
            }
            return (true);
        }

        /// <summary>
        /// 円ハフ検出実行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 円ハフ検出処理実行関数
        /// </remarks>
        private bool fnCircleHoughExec()
        {
            bool Result = false;
            try
            {
                if (comboBoxSelectFilter.SelectedIndex == 0)
                {
                    //画像設定
                    m_houghcorrelation.SrcImages[0] = m_image;
                    //時間測定開始
                    m_timer.Start();
                    //相関エッジ法による２次元エッジ抽出を行い、円ハフ検出処理実行
                    m_houghcorrelation.Execute();
                    //時間測定終了
                    msec = m_timer.Stop();
                }
                else
                {
                    //画像設定
                    m_houghsobel.SrcImages[0] = m_image;
                    //時間測定開始
                    m_timer.Start();
                    //ソーベルエッジ法による２次元エッジ抽出を行い、円ハフ検出処理実行
                    m_houghsobel.Execute();
                    //時間測定終了
                    msec = m_timer.Stop();
                }
                //測定時間表示
                textBoxTime.Text = msec.ToString("0.000");
                Result = true;
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("サーチエラーが発生しました。ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return (Result);
        }

        /// <summary>
        /// 結果表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 結果表示関数
        /// </remarks>
        private void fnResultDisplay()
        {
            int i;
            int count;
            FVIL.Hough.CFviCircleHoughData data;

            //検出したデータの総数を取得
            if (comboBoxSelectFilter.SelectedIndex == 0)
            {
                //相関エッジ
                count = m_houghcorrelation.Result.Count;
            }
            else
            {
                //ソーベルエッジ
                count = m_houghsobel.Result.Count;
            }

            for (i = 0; i < count; i++)
            {
                if (comboBoxSelectFilter.SelectedIndex == 0)
                {
                    //相関エッジ
                    data = m_houghcorrelation.Result[i];
                }
                else
                {
                    //ソーベルエッジ
                    data = m_houghsobel.Result[i];
                }

                //円データ生成
                FVIL.GDI.CFviGdiCircle ol_circle = new FVIL.GDI.CFviGdiCircle(data.ToCFviCircle());
                ol_circle.Pen.Color = System.Drawing.Color.Red;

                //中心
                FVIL.GDI.CFviGdiPoint ol_center = new FVIL.GDI.CFviGdiPoint();
                ol_center.Position = data.Center;
                ol_center.Pen.Color = System.Drawing.Color.Red;
                ol_center.Style = FVIL.GDI.FigureStyle.Cross;
                ol_center.Size = new System.Drawing.Size(5, 5);

                //番号
                FVIL.GDI.CFviGdiString ol_no = new FVIL.GDI.CFviGdiString();
                ol_no.Text = i.ToString();
                ol_no.Position = data.Center;
                ol_no.Align = FVIL.GDI.TextAlign.Right | FVIL.GDI.TextAlign.Bottom;
                ol_no.Color = System.Drawing.Color.Red;

                //追加
                m_overlay.Figures.Add(ol_circle);
                m_overlay.Figures.Add(ol_center);
                m_overlay.Figures.Add(ol_no);

                //リスト表示
                String[] listitem = {i.ToString(), data.Center.X.ToString("0.00"), data.Center.Y.ToString("0.00"), data.Radius.ToString("0.00"), data.Score.ToString()};
                listViewResult.Items.Add(new ListViewItem(listitem));
            }
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// 円ハフエッジ検出手法選択コンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 相関エッジ・ソーベルエッジの選択。
        /// </remarks>
        private void comboBoxSelectFilter_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxSelectFilter.SelectedIndex == 0)
            {
                //相関エッジ用パラメータ表示
                numericUpDownRegionHeight.Visible = true;
                numericUpDownRegionWidth.Visible = true;
                textBoxSigmoidK.Visible = true;
                textBoxVarThreshold.Visible = true;
                labelRegionWidth.Visible = true;
                labelRegionHeight.Visible = true;
                labelSigmoidK.Visible = true;
                labelVarThreshold.Visible = true;
                labelCondition1.Visible = true;
                labelCondition2.Visible = true;
                labelCondition3.Visible = true;
                //相関エッジ用パラメータ初期値設定
                numericUpDownEdgeThreshold.Maximum = 255;
                numericUpDownEdgeThreshold.Value = (decimal)m_correlationparam.EdgeThreshold;
                numericUpDownNmsLength.Value = (decimal)m_correlationparam.NmsLength;
            }
            else
            {
                //相関エッジ用パラメータ非表示
                numericUpDownRegionHeight.Visible = false;
                numericUpDownRegionWidth.Visible = false;
                textBoxSigmoidK.Visible = false;
                textBoxVarThreshold.Visible = false;
                labelRegionWidth.Visible = false;
                labelRegionHeight.Visible = false;
                labelSigmoidK.Visible = false;
                labelVarThreshold.Visible = false;
                labelCondition1.Visible = false;
                labelCondition2.Visible = false;
                labelCondition3.Visible = false;
                //ソーベルエッジ用パラメータ初期値設定
                numericUpDownEdgeThreshold.Maximum = 1023;
                numericUpDownEdgeThreshold.Value = (decimal)m_sobelparam.EdgeThreshold;
                numericUpDownNmsLength.Value = (decimal)m_sobelparam.NmsLength;
            }
        }

        /// <summary>
        /// 画像エッジ表示チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像のエッジの表示を切替えます。
        /// </remarks>
        private void checkBoxEdgeDisplay_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (checkBoxEdgeDisplay.Checked == true)
                {
                    List<FVIL.Edge.CFviEdgeData> edges_image;

                    //描画クリア
                    m_overlayEdge.Figures.Clear();
                    //パラメータ設定
                    fnCircleHoughPraramSet();

                    if (comboBoxSelectFilter.SelectedIndex == 0)
                    {
                        //画像設定
                        m_houghcorrelation.SrcImages[0] = m_image;
                        //円ハフ検出実行(相関エッジ法)
                        m_houghcorrelation.Execute();
                        //処理対象画像のエッジ点
                        edges_image = m_houghcorrelation.GetEdges();
                    }
                    else
                    {
                        //画像設定
                        m_houghsobel.SrcImages[0] = m_image;
                        //円ハフ検出実行(ソーベルエッジ法)
                        m_houghsobel.Execute();
                        //処理対象画像のエッジ点
                        edges_image = m_houghsobel.GetEdges();
                    }

                    //エッジの表示
                    for (int i = 0; i < edges_image.Count; i++)
                    {
                        FVIL.Edge.CFviEdgeData edge = edges_image[i];
                        // エッジ位置に ＞ マークを表示 (0度が左向き)
                        FVIL.GDI.CFviGdiPoint position = new FVIL.GDI.CFviGdiPoint(edge.Position);
                        position.Size = new System.Drawing.Size(3, 1);
                        position.Style = FVIL.GDI.FigureStyle.Arrow;
                        position.Angle = edges_image[i].Angle;
                        position.Pen.Color = System.Drawing.Color.FromArgb(0, 0xFF, 0);
                        m_overlayEdge.Figures.Add(position);
                    }
                }
                else
                {
                    m_overlayEdge.Figures.Clear();
                }
                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 処理範囲設定チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 処理範囲の表示を切替えます。
        /// </remarks>
        private void checkBoxSetArea_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxSetArea.Checked == true)
            {
                //処理範囲設定用オーバーレイ表示。
                cFviImageView1.Display.OverlayProcarea.Active = true;
                //再描画
                cFviImageView1.Refresh();
            }
            else
            {
                //処理範囲設定用オーバーレイ非表示。
                cFviImageView1.Display.OverlayProcarea.Active = false;
                //再描画
                cFviImageView1.Refresh();
            }
        }

        /// <summary>
        /// x１ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void buttonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。

            //表示倍率の設定
            cFviImageView1.Display.Magnification = 1.0;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void buttonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            cFviImageView1.FitImageSize();
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void buttonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void buttonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if (Value <= 0)
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void checkBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void buttonSaveImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                    //保存するファイル形式が固定の場合は、以下のように記述します。
                    //例1) bmpファイルの場合
                    //FVIL.File.CFviImageFileBmp file = new FVIL.File.CFviImageFileBmp();
                    //file.Save(ofdlg.FileName, m_image);

                    //例2) jpgファイルの場合
                    //FVIL.File.CFviImageFileJpg file = new FVIL.File.CFviImageFileJpg();
                    //file.Save(ofdlg.FileName, m_image);
                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch ((ex.ErrorCode))
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FPM_1CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ビデオデバイスがオープンされていた場合はクローズする
            if (m_flgVideoOpened) 
            {
                m_video.Close();
                m_flgVideoOpened = false;
            }
        }
        
        /// <summary>
        /// シグモイド関数Kテキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// シグモイド関数のパラメータkのエラー検出。
        /// </remarks>
        private void textBoxSigmoidK_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_correlationparam.SigmoidK = double.Parse(textBoxSigmoidK.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxSigmoidK.Text = "1.0";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 分散閾値テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 分散閾値のエラー検出。
        /// </remarks>
        private void textBoxVarThreshold_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_correlationparam.VarThreshold = double.Parse(textBoxVarThreshold.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxVarThreshold.Text = "25.0";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 分解能テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 分解能のエラー検出。
        /// </remarks>
        private void textBoxResolution_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (comboBoxSelectFilter.SelectedIndex == 0)
                {
                    //相関エッジ
                    m_houghcorrelation.Param.Resolution = double.Parse(textBoxResolution.Text.ToString());
                }
                else
                {
                    //ソーベルエッジ
                    m_houghsobel.Param.Resolution = double.Parse(textBoxResolution.Text.ToString());
                }
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxVarThreshold.Text = "1.0";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 円の最小半径テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 円の最小半径のエラー検出。
        /// </remarks>
        private void textBoxRadiusMin_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (comboBoxSelectFilter.SelectedIndex == 0)
                {
                    //相関エッジ
                    m_houghcorrelation.Param.RadiusMin = double.Parse(textBoxRadiusMin.Text.ToString());
                }
                else
                {
                    //ソーベルエッジ
                    m_houghsobel.Param.RadiusMin = double.Parse(textBoxRadiusMin.Text.ToString());
                }
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxVarThreshold.Text = "20.0";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 円の最大半径テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 円の最大半径のエラー検出。
        /// </remarks>
        private void textBoxRadiusMax_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (comboBoxSelectFilter.SelectedIndex == 0)
                {
                    //相関エッジ
                    m_houghcorrelation.Param.RadiusMax = double.Parse(textBoxRadiusMax.Text.ToString());
                }
                else
                {
                    //ソーベルエッジ
                    m_houghsobel.Param.RadiusMax = double.Parse(textBoxRadiusMax.Text.ToString());
                }
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxVarThreshold.Text = "40.0";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// リストのクリックに応じて表示を変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// リストのアイテムへのクリックに応じてオーバレイの表示を変更。
        /// </remarks>
        private void listViewResult_MouseUp(object sender, MouseEventArgs e)
        {
            int index = 0;

            //選択したリストのみオーバーレイを表示する、リスト以外を選択した場合全オーバーレイを表示する。
            for (int i = 0; i < listViewResult.Items.Count; i++)
            {
                ListViewItem item = listViewResult.Items[i];
                bool bEnable;

                //リスト内を選択したか判定
                if (listViewResult.SelectedItems.Count > 0)
                {
                    bEnable = (item.Checked || item.Selected);      //選択したアイテムの場合TRUEを設定し、それ以外はFALSEを設定する
                }
                else
                {
                    bEnable = true;                                 //リスト以外を選択した場合全てTRUEを設定する
                }

                //選択したアイテムを表示（未表示）にする
                m_overlay.Figures[index].Enable = bEnable;
                m_overlay.Figures[index + 1].Enable = bEnable;
                m_overlay.Figures[index + 2].Enable = bEnable;
                index = index + 3;
            }
            //再描画
            cFviImageView1.Refresh();
        }

    }
}